#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "term.h"

void term_init(struct term *t, int rows, int cols)
{
	t->state = TERM_STATE_NORMAL;
	t->cursor.x = 0;
	t->cursor.y = 0;
	t->rows = rows;
	t->cols = cols;
	t->lines = calloc(t->rows, sizeof(char *));
	for (int i = 0; i < t->rows; i++) {
		t->lines[i] = calloc(t->cols, sizeof(char *));
		memset(t->lines[i], ' ', t->cols);
	}
}

void term_put_string(struct term *t, char *str)
{
	for (int i = 0; i < strlen(str); i++) {
		term_put(t, str[i]);
	}
}

void term_cursor_inc_y(struct term *t)
{
	if (t->cursor.y == t->rows - 1) {
		for (int i = 1; i < t->rows; i++) {
			memcpy(t->lines[i-1], t->lines[i], t->cols);
		}
		memset(t->lines[t->cursor.y], ' ', t->cols);
	} else {
		t->cursor.y++;
	}
}

void term_put(struct term *t, char c)
{
	if (t->state == TERM_STATE_NORMAL) {
		switch (c) {
			case '\n':
				term_cursor_inc_y(t);
				t->cursor.x = 0;
				break;
			case '\r':
				t->cursor.x = 0;
				break;
			case '\b':
				t->cursor.x--;
				break;
			case '\x1b':
				//t->state = TERM_STATE_ESC;
				break;
			default:
				t->lines[t->cursor.y][t->cursor.x] = c;
				if (t->cursor.x == t->cols - 1) {
					term_cursor_inc_y(t);
					t->cursor.x = 0;
				} else {
					t->cursor.x++;
				}
		}
	} else if (t->state == TERM_STATE_ESC) {
		switch (c) {
		}
	}
}

char *term_to_string(struct term *t)
{
	int str_size = t->rows * t->cols + t->rows;
	char *str = calloc(str_size, sizeof(char));
	int str_idx = 0;

	for (int i = 0; i < t->rows; i++) {
		for (int j = 0; j < t->cols && str_idx < str_size; j++) {
			str[str_idx++] = t->lines[i][j];
		}
		if (i == t->rows - 1) {
			str[str_idx++] = '\0';
		} else {
			str[str_idx++] = '\n';
		}
	}

	return str;
}

void term_free(struct term *t)
{
	for(int i = 0; i < t->rows; i++) {
		if (t->lines[i] != NULL) {
			free(t->lines[i]);
		}
	}
	free(t->lines);
}
